@@ -97,7 +97,7 @@ inline static void clean_up_thread_list(NewGC *gc)
 
       if(prev) prev->next = next;
       if(!prev) gc->thread_infos = next;
-      free(work);
+      ofm_free(work, sizeof(GC_Thread_Info));
       work = next;
     }
   }
@@ -175,7 +175,7 @@ void BTC_register_root_custodian(void *_c)
 
   if (gc->owner_table) {
     /* Reset */
-    free(gc->owner_table);
+    ofm_free(gc->owner_table, sizeof(OTEntry*) * gc->owner_table_size);
     gc->owner_table = NULL;
     gc->owner_table_size = 0;
   }
@@ -214,7 +214,7 @@ inline static void free_owner_set(NewGC *gc, int set)
 {
   OTEntry **owner_table = gc->owner_table;
   if(owner_table[set]) {
-    free(owner_table[set]);
+    ofm_free(owner_table[set], sizeof(OTEntry));
   }
   owner_table[set] = NULL;
 }
@@ -251,7 +251,7 @@ inline static uintptr_t custodian_usage(NewGC*gc, void *custodian)
       CHECK_PARK_UNUSED(gc);
       gc->park[0] = custodian;
       gc->really_doing_accounting = 1;
-      garbage_collect(gc, 1, 0, NULL);
+      garbage_collect(gc, 1, 0, 0, NULL);
       custodian = gc->park[0]; 
       gc->park[0] = NULL;
     }
@@ -268,15 +268,85 @@ inline static uintptr_t custodian_usage(NewGC*gc, void *custodian)
   return gcWORDS_TO_BYTES(retval);
 }
 
+#ifdef MZ_USE_PLACES
+
+static mzrt_mutex *master_btc_lock;
+static mzrt_sema *master_btc_sema;
+static int master_btc_lock_count = 0;
+static int master_btc_lock_waiters = 0;
+
+void init_master_btc_locks()
+{
+  mzrt_mutex_create(&master_btc_lock);
+  mzrt_sema_create(&master_btc_sema, 0);
+}
+
+static void check_master_btc_mark(NewGC *gc, mpage *page)
+{
+  if (!gc->master_page_btc_mark_checked) {
+    int pause = 1;
+    RELEASE_PAGE_LOCK(1, page);
+    while (pause) {
+      mzrt_mutex_lock(master_btc_lock);
+      if (master_btc_lock_count
+          && (gc->new_btc_mark != MASTERGC->new_btc_mark)) {
+        pause = 1;
+        master_btc_lock_waiters++;
+      } else {
+        pause = 0;
+        MASTERGC->new_btc_mark = gc->new_btc_mark;
+        master_btc_lock_count++;
+      }
+      mzrt_mutex_unlock(master_btc_lock);
+
+      if (pause)
+        mzrt_sema_wait(master_btc_sema);
+    }
+    TAKE_PAGE_LOCK(1, page);
+    gc->master_page_btc_mark_checked = 1;
+  }
+}
+
+static void release_master_btc_mark(NewGC *gc)
+{
+  if (gc->master_page_btc_mark_checked) {
+    /* release the lock on the master's new_btc_mark value */
+    mzrt_mutex_lock(master_btc_lock);
+    --master_btc_lock_count;
+    if (!master_btc_lock_count && master_btc_lock_waiters) {
+      --master_btc_lock_waiters;
+      mzrt_sema_post(master_btc_sema);
+    }
+    mzrt_mutex_unlock(master_btc_lock);
+  }
+}
+
+#else
+
+static void check_master_btc_mark(NewGC *gc, mpage *page) { }
+static void release_master_btc_mark(NewGC *gc) { }
+
+#endif
+
 inline static void BTC_memory_account_mark(NewGC *gc, mpage *page, void *ptr, int is_a_master_page)
 {
   GCDEBUG((DEBUGOUTF, "BTC_memory_account_mark: %p/%p\n", page, ptr));
 
   /* In the case of is_a_master_page, whether this place is charged is
-     a little random: there's no guarantee that the btc_mark values are
-     in sync, and there are races among places. Approximations are ok for
-     accounting, though, as long as the probably for completely wrong
-     accounting is very low. */
+     a little random: there's no guarantee that the btc_mark values
+     are in sync, and there are races among places. Approximations are
+     ok for accounting, though, as long as the probably for completely
+     wrong accounting is very low.
+
+     At the same time, we need to synchronize enough so that two
+     places with different new_btc_mark values don't send each other
+     into infinite loops (with the btc_mark value bouncing back and
+     forth) or overcounting. We synchronize enough by having a single
+     new_btc_mark value for master pages, and we stall if the value
+     isn't what this place wants. */
+
+  if (is_a_master_page)
+    check_master_btc_mark(gc, page);
 
   if(page->size_class) {
     if(page->size_class > 1) {
@@ -368,11 +438,17 @@ int BTC_bi_chan_mark(void *p, struct NewGC *gc)
 {
   if (gc->doing_memory_accounting) {
     Scheme_Place_Bi_Channel *bc = (Scheme_Place_Bi_Channel *)p;
-    /* Race conditions here on `mem_size', and likely double counting
-       when the same async channels are accessible from paired bi
-       channels --- but those approximations are ok for accounting. */
-    account_memory(gc, gc->current_mark_owner, bc->link->sendch->mem_size, 0);
-    account_memory(gc, gc->current_mark_owner, bc->link->recvch->mem_size, 0);
+    /* The `link` field can be NULL if the channel is still being
+       set up: */
+    if (bc->link) {
+      /* Race conditions here on `mem_size', and likely double counting
+         when the same async channels are accessible from paired bi
+         channels --- but those approximations are ok for accounting. */
+      if (bc->link->sendch)
+        account_memory(gc, gc->current_mark_owner, gcBYTES_TO_WORDS(bc->link->sendch->mem_size), 0);
+      if (bc->link->recvch)
+        account_memory(gc, gc->current_mark_owner, gcBYTES_TO_WORDS(bc->link->recvch->mem_size), 0);
+    }
   }
   return gc->mark_table[btc_redirect_bi_chan](p, gc);
 }
@@ -387,11 +463,10 @@ static void btc_overmem_abort(NewGC *gc)
 static void propagate_accounting_marks(NewGC *gc)
 {
   void *p;
-  Mark2_Proc *mark_table = gc->mark_table;
 
   while(pop_ptr(gc, &p) && !gc->kill_propagation_loop) {
     /* GCDEBUG((DEBUGOUTF, "btc_account: popped off page %p:%p, ptr %p\n", page, page->addr, p)); */
-    propagate_marks_worker(gc, mark_table, p); 
+    propagate_marks_worker(gc, p);
   }
   if(gc->kill_propagation_loop)
     reset_pointer_stack(gc);
@@ -428,6 +503,7 @@ static void BTC_do_accounting(NewGC *gc)
     gc->doing_memory_accounting = 1;
     gc->in_unsafe_allocation_mode = 1;
     gc->unsafe_allocation_abort = btc_overmem_abort;
+    gc->master_page_btc_mark_checked = 0;
 
     /* clear the memory use numbers out */
     for(i = 1; i < table_size; i++)
@@ -469,6 +545,8 @@ static void BTC_do_accounting(NewGC *gc)
       gc->phantom_count = save_count;
     }
 
+    release_master_btc_mark(gc);
+
     /* walk backward folding totals int parent */
     cur = last;
     while (cur) {
@@ -507,7 +585,7 @@ inline static void BTC_add_account_hook(int type,void *c1,void *c2,uintptr_t b)
       gc->park[0] = c1; 
       gc->park[1] = c2;
       gc->really_doing_accounting = 1;
-      garbage_collect(gc, 1, 0, NULL);
+      garbage_collect(gc, 1, 0, 0, NULL);
       c1 = gc->park[0]; gc->park[0] = NULL;
       c2 = gc->park[1]; gc->park[1] = NULL;
     }
@@ -559,7 +637,7 @@ inline static void clean_up_account_hooks(NewGC *gc)
 
       if(prev) prev->next = next;
       if(!prev) gc->hooks = next;
-      free(work);
+      ofm_free(work, sizeof(AccountHook));
       work = next;
     }
   }
@@ -616,7 +694,7 @@ inline static void BTC_run_account_hooks(NewGC *gc)
       if(prev) prev->next = next;
       if(!prev) gc->hooks = next;
       scheme_schedule_custodian_close(work->c2);
-      free(work);
+      ofm_free(work, sizeof(AccountHook));
       work = next;
     } else {
       prev = work; 
