@@ -13,32 +13,40 @@ typedef struct mpage {
   struct mpage *next;
   struct mpage *prev;
   void *addr;
-  uintptr_t previous_size; /* for med page, place to search for available block; for jit nursery, allocated size */
-  uintptr_t size; /* big page size, med page element size, or nursery starting point */
-/*
-  unsigned char generation    :1;
-  unsigned char back_pointers :1;
-  unsigned char size_cless    :2;
-  unsigned char page_type     :3; 
-  unsigned char marked_on     :1;
-  unsigned char has_new       :1;
-  unsigned char mprotected    :1;
-  unsigned char added         :1;
-*/
-  unsigned char generation    ;
-  unsigned char back_pointers ;
-  unsigned char size_class    ; /* 0 => small; 1 => med; 2 => big; 3 => big marked */
-  unsigned char page_type     ; 
-  unsigned char marked_on     ;
-  unsigned char has_new       ;
-  unsigned char mprotected    ;
-  unsigned char added         ;
-  unsigned short live_size;
+  void *mmu_src_block;
+  struct mpage *modified_next; /* next in chain of pages for backpointers, marks, etc. */
+  struct mpage *reprotect_next; /* next in a chain of pages that need to be re-protected */
 #ifdef MZ_GC_BACKTRACE
   void **backtrace;
   void *backtrace_page_src;
 #endif
-  void *mmu_src_block;
+#ifdef MZ_USE_PLACES
+  uintptr_t page_lock; /* for master GC pages during marking */
+#endif
+  /* The `size` field is overleaded for related meanings:
+       - big page => the size of the allocated object
+       - small page, nursery, gen-1/2 => offset for next allocate = allocated bytes + PREFIX_SIZE
+     For a medium page, the `obj_size` field is used instead. */
+  union {
+    uintptr_t size; /* size and/or allocation offset (see above) */
+    uintptr_t obj_size; /* size of each object on a medium page */
+  };
+  union {
+    uintptr_t alloc_size; /* for a nursery: total size of the nursery */
+    uintptr_t med_search_start; /* medium page: offset for searching for a free slot */
+    uintptr_t scan_boundary; /* small gen1 page: during GC, boundary between objects that can be
+                                left alone and that that will be scanned & fixed up; objects before
+                                have cleared "mark" bits, while objects after (may) have "mark" bits sets */
+  };
+  unsigned short live_size; /* except for big pages, total size of live objects on the page */
+  unsigned char generation    :2;
+  unsigned char back_pointers :1;
+  unsigned char size_class    :2;
+  unsigned char page_type     :3; 
+  unsigned char marked_on     :1;
+  unsigned char marked_from   :1;
+  unsigned char has_new       :1;
+  unsigned char mprotected    :1;
 } mpage;
 
 typedef struct Gen0 {
@@ -50,6 +58,12 @@ typedef struct Gen0 {
   uintptr_t page_alloc_size;
 } Gen0;
 
+typedef struct Gen_Half {
+  struct mpage *curr_alloc_page;
+  struct mpage *pages;
+  struct mpage *old_pages;
+} Gen_Half;
+
 typedef struct MsgMemory {
   struct mpage *pages;
   struct mpage *big_pages;
@@ -106,13 +120,6 @@ typedef struct Page_Range {
   uintptr_t range_alloc_used;
 } Page_Range;
 
-typedef struct {
-  char *start;
-  intptr_t len;
-  short age;
-  short zeroed;
-} AllocCacheBlock;
-
 #ifdef MZ_USE_PLACES
 typedef struct NewGCMasterInfo {
   uintptr_t size;
@@ -135,6 +142,7 @@ typedef mpage **PageMap;
 
 typedef struct NewGC {
   Gen0 gen0;
+  Gen_Half gen_half;
   Mark2_Proc  *mark_table;   /* the table of mark procs */
   Fixup2_Proc *fixup_table;  /* the table of repair procs */
   PageMap page_maps;
@@ -145,12 +153,23 @@ typedef struct NewGC {
   struct mpage *med_pages[MED_PAGE_TYPES][NUM_MED_PAGE_SIZES];
   struct mpage *med_freelist_pages[MED_PAGE_TYPES][NUM_MED_PAGE_SIZES];
 
+  intptr_t num_gen1_pages;
+
+  /* linked list of pages with back pointers to be traversed in a
+     minor collection, etc.: */
+  struct mpage *modified_next;
+  /* linked list of pages that need to be given write protection at
+     the end of the GC cycle: */
+  struct mpage *reprotect_next;
+
   MarkSegment *mark_stack;
 
   /* Finalization */
   Fnl *run_queue;
   Fnl *last_in_queue;
 
+  int mark_depth;
+
   struct NewGC *primoridal_gc;
   uintptr_t max_heap_size;
   uintptr_t max_pages_in_heap;
@@ -160,6 +179,7 @@ typedef struct NewGC {
   void (*unsafe_allocation_abort)(struct NewGC *);
   uintptr_t memory_in_use; /* the amount of memory in use */
   uintptr_t pending_msg_size; /* set in master, only */
+  uintptr_t prev_pending_msg_size; /* set in master, only */
 
   /* blame the child thread infos */
   GC_Thread_Info *thread_infos;
@@ -173,7 +193,10 @@ typedef struct NewGC {
   unsigned char full_needed_for_finalization :1;
   unsigned char no_further_modifications     :1;
   unsigned char gc_full                      :1; /* a flag saying if this is a full/major collection */
+  unsigned char use_gen_half                 :1;
   unsigned char running_finalizers           :1;
+  unsigned char back_pointers                :1;
+  unsigned char need_fixup                   :1;
 
   /* blame the child */
   unsigned int doing_memory_accounting        :1;
@@ -187,6 +210,7 @@ typedef struct NewGC {
   OTEntry **owner_table;
   unsigned int owner_table_size;
   AccountHook *hooks;
+  int master_page_btc_mark_checked;
 
   uintptr_t number_of_gc_runs;
   unsigned int since_last_full;
@@ -194,8 +218,13 @@ typedef struct NewGC {
 
   /* These collect information about memory usage, for use in GC_dump. */
   uintptr_t peak_memory_use;
+  uintptr_t peak_pre_memory_use;
   uintptr_t num_minor_collects;
   uintptr_t num_major_collects;
+
+  uintptr_t minor_old_traversed;
+  uintptr_t minor_old_skipped;
+  uintptr_t modified_unprotects;
   
   /* THREAD_LOCAL variables that need to be saved off */
   MarkSegment  *saved_mark_stack;
@@ -220,9 +249,10 @@ typedef struct NewGC {
 
   GC_Immobile_Box *immobile_boxes;
 
-  /* Common with CompactGC */
   Fnl *finalizers;
   Fnl *splayed_finalizers;
+  Fnl *gen0_finalizers;
+  Fnl *splayed_gen0_finalizers;
   int num_fnls;
 
   void *park[2];
@@ -236,6 +266,7 @@ typedef struct NewGC {
   unsigned short phantom_tag;
 
   uintptr_t phantom_count;
+  uintptr_t gen0_phantom_count;
 
   Roots roots;
   GC_Weak_Array *weak_arrays;
