@@ -17,7 +17,7 @@ enum {
 };
 
 #if defined(_WIN32) || defined(__CYGWIN32__)
-/* No block cache or alloc cache */
+/* No block cache or alloc cache; relies on APAGE_SIZE matching allocator's alignment */
 #elif defined(OSKIT)
 # define OS_ALLOCATOR_NEEDS_ALIGNMENT
 #elif defined(MZ_USE_PLACES) || defined(PREFER_MMAP_LARGE_BLOCKS)
@@ -28,7 +28,15 @@ enum {
 
 #ifdef USE_BLOCK_CACHE
 # define USE_ALLOC_CACHE
+# define QUEUED_MPROTECT_INFECTS_SMALL 1
+#else
+# define QUEUED_MPROTECT_INFECTS_SMALL 0
 #endif
+#define QUEUED_MPROTECT_INFECTS_MED 0
+
+/* Either USE_ALLOC_CACHE or OS_ALLOCATOR_NEEDS_ALIGNMENT must be
+   enabled, unless the lower-level allocator's alignment matches
+   APAGE_SIZE. */
 
 struct AllocCacheBlock;
 struct BlockCache;
@@ -78,7 +86,7 @@ static inline size_t mmu_round_up_to_os_page_size(MMU *mmu, size_t len) {
 
 static inline void mmu_assert_os_page_aligned(MMU *mmu, size_t p) {
   if (p & (mmu->os_pagesize - 1)) {
-    printf("address or size is not OS PAGE ALIGNED!!!!");
+    GCPRINT(GCOUTF, "address or size is not page-aligned\n");
     abort();
   }
 }
@@ -193,10 +201,14 @@ static int mmu_should_compact_page(MMU *mmu, void **src_block) {
   return 0;
 }
 
-static void mmu_write_unprotect_page(MMU *mmu, void *p, size_t len) {
+static void mmu_write_unprotect_page(MMU *mmu, void *p, size_t len, int type, void **src_block) {
   mmu_assert_os_page_aligned(mmu, (size_t)p);
   mmu_assert_os_page_aligned(mmu, len);
+#ifdef USE_BLOCK_CACHE
+  block_cache_protect_one_page(mmu->block_cache, p, len, type, 1, src_block);
+#else
   os_protect_pages(p, len, 1);
+#endif
 }
 
 static void mmu_queue_protect_range(MMU *mmu, void *p, size_t len, int type, int writeable, void **src_block) {
