@@ -1,6 +1,6 @@
 /*
   Racket
-  Copyright (c) 2004-2014 PLT Design Inc.
+  Copyright (c) 2004-2015 PLT Design Inc.
   Copyright (c) 1995-2000 Matthew Flatt
 
     This library is free software; you can redistribute it and/or
@@ -31,7 +31,9 @@
    (except for the garbage collector, which is in `gc', `sgc', or
    `gc2', depending on which one you're using). */
 
-#define __MINGW32_DELAY_LOAD__ 1
+#ifdef __MINGW32__
+# define __MINGW32_DELAY_LOAD__ 1
+#endif
 #include "scheme.h"
 
 /*========================================================================*/
@@ -161,7 +163,7 @@ extern Scheme_Object *scheme_initialize(Scheme_Env *env);
 /*                            OS process name                             */
 /*========================================================================*/
 
-#if defined(linux)
+#if defined(__linux__)
 # include <sys/prctl.h>
 # ifdef PR_SET_NAME
 #  define CAN_SET_OS_PROCESS_NAME 1
@@ -290,13 +292,8 @@ START_XFORM_SKIP;
 # include "parse_cmdl.inc"
 #endif
 
-#ifdef IMPLEMENT_THREAD_LOCAL_VIA_WIN_TLS
-extern intptr_t _tls_index;
-# ifdef __MINGW32__
-static __thread void *tls_space;
-# else
-static __declspec(thread) void *tls_space;
-# endif
+#ifdef DOS_FILE_SYSTEM
+# include "win_tls.inc"
 #endif
 
 #ifdef DOS_FILE_SYSTEM
@@ -304,31 +301,16 @@ void load_delayed()
 {
   (void)SetErrorMode(SEM_FAILCRITICALERRORS);
 
+# ifndef MZ_NO_LIBRACKET_DLL
   /* Order matters: load dependencies first */
-# ifndef MZ_PRECISE_GC
+#  ifndef MZ_PRECISE_GC
   load_delayed_dll(NULL, "libmzgcxxxxxxx.dll");
-# endif
+#  endif
   load_delayed_dll(NULL, "libracket" DLL_3M_SUFFIX "xxxxxxx.dll");
-  record_dll_path();
-# ifdef IMPLEMENT_THREAD_LOCAL_VIA_WIN_TLS
-#  ifdef __MINGW32__
-  {
-    /* gcc declares space for the thread-local variable in a way that
-       the OS can set up, but its doesn't actually map variables
-       through the OS-supplied mechanism. Just assume that the first
-       thread-local variable is ours. */
-    void **base;
-#  ifdef _WIN64
-    asm("mov %%gs:(0x58), %0;" :"=r"(base));
-#  else
-    asm("mov %%fs:(0x2C), %0;" :"=r"(base));
-#  endif
-    scheme_register_tls_space(*base, _tls_index);
-  }
-#  else
-  scheme_register_tls_space(&tls_space, _tls_index);
-#  endif
 # endif
+  record_dll_path();
+
+  register_win_tls();
 }
 #endif
 
@@ -495,14 +477,6 @@ static void do_scheme_rep(Scheme_Env *env, FinishArgs *fa)
   }
 }
 
-/*========================================================================*/
-/*                         win32 manifest                                 */
-/*========================================================================*/
-
-#if _MSC_VER >= 1400
-#pragma comment(linker,"/manifestdependency:\"type='win32' name='Microsoft.Windows.Common-Controls' version='6.0.0.0' processorArchitecture='*' publicKeyToken='6595b64144ccf1df' language='*'\"")
-#endif
-
 /*========================================================================*/
 /*                         junk for testing                               */
 /*========================================================================*/
