@@ -1,6 +1,6 @@
 /*
   Racket
-  Copyright (c) 2004-2014 PLT Design Inc.
+  Copyright (c) 2004-2015 PLT Design Inc.
   Copyright (c) 1995-2001 Matthew Flatt
 
     This library is free software; you can redistribute it and/or
@@ -223,6 +223,11 @@ Thread_Local_Variables *scheme_external_get_thread_local_variables() XFORM_SKIP_
 {
   return scheme_get_thread_local_variables();
 }
+#else
+void scheme_register_tls_space(void *tls_space, int tls_index) XFORM_SKIP_PROC
+{
+  /* Nothing to do; provided for compatibility. */
+}
 #endif
 
 #ifdef IMPLEMENT_THREAD_LOCAL_EXTERNALLY_VIA_PROC
@@ -1633,6 +1638,14 @@ void scheme_collect_garbage(void)
   GC_gcollect();
 }
 
+void scheme_collect_garbage_minor(void)
+{
+#ifdef MZ_PRECISE_GC
+  GC_gcollect_minor();
+#else
+#endif
+}
+
 void scheme_enable_garbage_collection(int on)
 {
 #ifdef MZ_PRECISE_GC
