@@ -1,6 +1,6 @@
 /*
   Racket
-  Copyright (c) 2004-2014 PLT Design Inc.
+  Copyright (c) 2004-2015 PLT Design Inc.
   Copyright (c) 1995-2001 Matthew Flatt
   All rights reserved.
 
@@ -42,6 +42,7 @@ MZ_EXTERN void scheme_longjmpup(Scheme_Jumpup_Buf *b);
 MZ_EXTERN void scheme_reset_jmpup_buf(Scheme_Jumpup_Buf *b);
 
 #ifdef USE_MZ_SETJMP
+MZ_EXTERN Scheme_Setjmp_Proc scheme_get_mz_setjmp(void);
 MZ_EXTERN int scheme_mz_setjmp(mz_pre_jmp_buf b);
 MZ_EXTERN void scheme_mz_longjmp(mz_pre_jmp_buf b, int v);
 #endif
@@ -200,12 +201,8 @@ MZ_EXTERN void scheme_set_can_break(int on);
 MZ_EXTERN void scheme_push_break_enable(Scheme_Cont_Frame_Data *cframe, int on, int pre_check);
 MZ_EXTERN void scheme_pop_break_enable(Scheme_Cont_Frame_Data *cframe, int post_check);
 
-MZ_EXTERN int scheme_with_stack_freeze(Scheme_Frozen_Stack_Proc wha_f, void *wha_data);
-MZ_EXTERN int scheme_frozen_run_some(Scheme_Frozen_Stack_Proc do_f, void *do_data, int run_msecs);
-MZ_EXTERN int scheme_is_in_frozen_stack();
-
-MZ_EXTERN Scheme_Object *scheme_abort_continuation_no_dws (Scheme_Object *pt, Scheme_Object *v);
-MZ_EXTERN Scheme_Object *scheme_call_with_composable_no_dws (Scheme_Object *proc, Scheme_Object *pt);
+MZ_EXTERN Scheme_Object *scheme_abort_continuation_no_dws(Scheme_Object *pt, Scheme_Object *v);
+MZ_EXTERN Scheme_Object *scheme_call_with_composable_no_dws(Scheme_Object *proc, Scheme_Object *pt);
 
 MZ_EXTERN Scheme_On_Atomic_Timeout_Proc scheme_set_on_atomic_timeout(Scheme_On_Atomic_Timeout_Proc p);
 
@@ -415,7 +412,6 @@ MZ_EXTERN void *GC_malloc_atomic(size_t size_in_bytes);
 MZ_EXTERN void *GC_malloc_one_tagged(size_t size_in_bytes);
 MZ_EXTERN void *GC_malloc_atomic_uncollectable(size_t size_in_bytes);
 MZ_EXTERN void *scheme_malloc_uncollectable(size_t size_in_bytes);
-MZ_EXTERN void *GC_malloc_array_tagged(size_t size_in_bytes);
 MZ_EXTERN void *GC_malloc_allow_interior(size_t size_in_bytes);
 MZ_EXTERN void *GC_malloc_atomic_allow_interior(size_t size_in_bytes);
 MZ_EXTERN void *GC_malloc_tagged_allow_interior(size_t size_in_bytes);
@@ -466,6 +462,7 @@ MZ_EXTERN void scheme_dont_gc_ptr(void *p);
 MZ_EXTERN void scheme_gc_ptr_ok(void *p);
 
 MZ_EXTERN void scheme_collect_garbage(void);
+MZ_EXTERN void scheme_collect_garbage_minor(void);
 MZ_EXTERN void scheme_enable_garbage_collection(int on);
 
 #ifdef MZ_PRECISE_GC
@@ -475,7 +472,7 @@ MZ_EXTERN void **GC_variable_stack;
 MZ_EXTERN void GC_register_traversers(short tag, Size_Proc size, Mark_Proc mark, Fixup_Proc fixup,
 				      int is_constant_size, int is_atomic);
 MZ_EXTERN void *GC_resolve(void *p);
-MZ_EXTERN void GC_mark(const void *p);
+MZ_EXTERN void GC_mark(void *p);
 MZ_EXTERN void GC_fixup(void *p);
 MZ_EXTERN void *GC_fixup_self(void *p);
 #endif
@@ -514,6 +511,7 @@ MZ_EXTERN Scheme_Hash_Table *scheme_clone_hash_table(Scheme_Hash_Table *ht);
 MZ_EXTERN void scheme_clear_hash_table(Scheme_Hash_Table *ht);
 
 MZ_EXTERN Scheme_Hash_Tree *scheme_make_hash_tree(int kind);
+MZ_EXTERN Scheme_Hash_Tree *scheme_make_hash_tree_set(int kind);
 MZ_EXTERN Scheme_Hash_Tree *scheme_hash_tree_set(Scheme_Hash_Tree *tree, Scheme_Object *key, Scheme_Object *val);
 MZ_EXTERN Scheme_Object *scheme_hash_tree_get(Scheme_Hash_Tree *tree, Scheme_Object *key);
 XFORM_NONGCING MZ_EXTERN Scheme_Object *scheme_eq_hash_tree_get(Scheme_Hash_Tree *tree, Scheme_Object *key);
@@ -1143,8 +1141,8 @@ MZ_EXTERN intptr_t scheme_equal_hash_key(Scheme_Object *o);
 MZ_EXTERN intptr_t scheme_equal_hash_key2(Scheme_Object *o);
 MZ_EXTERN intptr_t scheme_recur_equal_hash_key(Scheme_Object *o, void *cycle_data);
 MZ_EXTERN intptr_t scheme_recur_equal_hash_key2(Scheme_Object *o, void *cycle_data);
-MZ_EXTERN intptr_t scheme_eqv_hash_key(Scheme_Object *o);
-MZ_EXTERN intptr_t scheme_eqv_hash_key2(Scheme_Object *o);
+XFORM_NONGCING MZ_EXTERN intptr_t scheme_eqv_hash_key(Scheme_Object *o);
+XFORM_NONGCING MZ_EXTERN intptr_t scheme_eqv_hash_key2(Scheme_Object *o);
 
 MZ_EXTERN void scheme_set_type_equality(Scheme_Type type, 
                                         Scheme_Equal_Proc f,
@@ -1155,13 +1153,12 @@ MZ_EXTERN int scheme_recur_equal(Scheme_Object *obj1, Scheme_Object *obj2, void
 MZ_EXTERN Scheme_Object *scheme_build_list(int argc, Scheme_Object **argv);
 MZ_EXTERN Scheme_Object *scheme_build_list_offset(int argc, Scheme_Object **argv, int delta);
 
-MZ_EXTERN int scheme_is_list(Scheme_Object *obj1);
-MZ_EXTERN int scheme_list_length(Scheme_Object *list);
-MZ_EXTERN int scheme_proper_list_length(Scheme_Object *list);
+XFORM_NONGCING MZ_EXTERN int scheme_is_list(Scheme_Object *obj1);
+XFORM_NONGCING MZ_EXTERN int scheme_list_length(Scheme_Object *list);
+XFORM_NONGCING MZ_EXTERN int scheme_proper_list_length(Scheme_Object *list);
 
 MZ_EXTERN Scheme_Object *scheme_alloc_list(int size);
-MZ_EXTERN Scheme_Object *scheme_map_1(Scheme_Object *(*f)(Scheme_Object*),
-			    Scheme_Object *l);
+MZ_EXTERN Scheme_Object *scheme_map_1(Scheme_Object *(*f)(Scheme_Object*), Scheme_Object *l);
 
 MZ_EXTERN Scheme_Object *scheme_car(Scheme_Object *pair);
 MZ_EXTERN Scheme_Object *scheme_cdr(Scheme_Object *pair);
@@ -1195,6 +1192,7 @@ MZ_EXTERN intptr_t scheme_get_seconds(void);
 XFORM_NONGCING MZ_EXTERN intptr_t scheme_get_milliseconds(void);
 XFORM_NONGCING MZ_EXTERN double scheme_get_inexact_milliseconds(void);
 XFORM_NONGCING MZ_EXTERN intptr_t scheme_get_process_milliseconds(void);
+XFORM_NONGCING MZ_EXTERN intptr_t scheme_get_process_children_milliseconds(void);
 XFORM_NONGCING MZ_EXTERN intptr_t scheme_get_thread_milliseconds(Scheme_Object *thrd);
 
 MZ_EXTERN char *scheme_banner(void);
@@ -1229,5 +1227,5 @@ MZ_EXTERN void *scheme_register_process_global(const char *key, void *val);
 MZ_EXTERN Scheme_Object *scheme_malloc_key(void);
 MZ_EXTERN void scheme_free_key(Scheme_Object *k);
 
-MZ_EXTERN void* scheme_jit_find_code_end(void *p);
+MZ_EXTERN void *scheme_jit_find_code_end(void *p);
 MZ_EXTERN void scheme_jit_now(Scheme_Object *f);
