@@ -3,6 +3,7 @@
 #define scheme_longjmpup (scheme_extension_table->scheme_longjmpup)
 #define scheme_reset_jmpup_buf (scheme_extension_table->scheme_reset_jmpup_buf)
 #ifdef USE_MZ_SETJMP
+#define scheme_get_mz_setjmp (scheme_extension_table->scheme_get_mz_setjmp)
 #define scheme_mz_setjmp (scheme_extension_table->scheme_mz_setjmp)
 #define scheme_mz_longjmp (scheme_extension_table->scheme_mz_longjmp)
 #endif
@@ -101,9 +102,6 @@
 #define scheme_set_can_break (scheme_extension_table->scheme_set_can_break)
 #define scheme_push_break_enable (scheme_extension_table->scheme_push_break_enable)
 #define scheme_pop_break_enable (scheme_extension_table->scheme_pop_break_enable)
-#define scheme_with_stack_freeze (scheme_extension_table->scheme_with_stack_freeze)
-#define scheme_frozen_run_some (scheme_extension_table->scheme_frozen_run_some)
-#define scheme_is_in_frozen_stack (scheme_extension_table->scheme_is_in_frozen_stack)
 #define scheme_abort_continuation_no_dws (scheme_extension_table->scheme_abort_continuation_no_dws)
 #define scheme_call_with_composable_no_dws (scheme_extension_table->scheme_call_with_composable_no_dws)
 #define scheme_set_on_atomic_timeout (scheme_extension_table->scheme_set_on_atomic_timeout)
@@ -225,7 +223,6 @@
 #define GC_malloc_one_tagged (scheme_extension_table->GC_malloc_one_tagged)
 #define GC_malloc_atomic_uncollectable (scheme_extension_table->GC_malloc_atomic_uncollectable)
 #define scheme_malloc_uncollectable (scheme_extension_table->scheme_malloc_uncollectable)
-#define GC_malloc_array_tagged (scheme_extension_table->GC_malloc_array_tagged)
 #define GC_malloc_allow_interior (scheme_extension_table->GC_malloc_allow_interior)
 #define GC_malloc_atomic_allow_interior (scheme_extension_table->GC_malloc_atomic_allow_interior)
 #define GC_malloc_tagged_allow_interior (scheme_extension_table->GC_malloc_tagged_allow_interior)
@@ -264,6 +261,7 @@
 #define scheme_dont_gc_ptr (scheme_extension_table->scheme_dont_gc_ptr)
 #define scheme_gc_ptr_ok (scheme_extension_table->scheme_gc_ptr_ok)
 #define scheme_collect_garbage (scheme_extension_table->scheme_collect_garbage)
+#define scheme_collect_garbage_minor (scheme_extension_table->scheme_collect_garbage_minor)
 #define scheme_enable_garbage_collection (scheme_extension_table->scheme_enable_garbage_collection)
 #ifdef MZ_PRECISE_GC
 # ifndef USE_THREAD_LOCAL
@@ -301,6 +299,7 @@
 #define scheme_clone_hash_table (scheme_extension_table->scheme_clone_hash_table)
 #define scheme_clear_hash_table (scheme_extension_table->scheme_clear_hash_table)
 #define scheme_make_hash_tree (scheme_extension_table->scheme_make_hash_tree)
+#define scheme_make_hash_tree_set (scheme_extension_table->scheme_make_hash_tree_set)
 #define scheme_hash_tree_set (scheme_extension_table->scheme_hash_tree_set)
 #define scheme_hash_tree_get (scheme_extension_table->scheme_hash_tree_get)
 #define scheme_eq_hash_tree_get (scheme_extension_table->scheme_eq_hash_tree_get)
@@ -704,6 +703,7 @@
 #define scheme_get_milliseconds (scheme_extension_table->scheme_get_milliseconds)
 #define scheme_get_inexact_milliseconds (scheme_extension_table->scheme_get_inexact_milliseconds)
 #define scheme_get_process_milliseconds (scheme_extension_table->scheme_get_process_milliseconds)
+#define scheme_get_process_children_milliseconds (scheme_extension_table->scheme_get_process_children_milliseconds)
 #define scheme_get_thread_milliseconds (scheme_extension_table->scheme_get_thread_milliseconds)
 #define scheme_banner (scheme_extension_table->scheme_banner)
 #define scheme_version (scheme_extension_table->scheme_version)
