@@ -1,6 +1,6 @@
 /*
   Racket
-  Copyright (c) 2004-2014 PLT Design Inc.
+  Copyright (c) 2004-2015 PLT Design Inc.
   Copyright (c) 1995-2001 Matthew Flatt
 
     This library is free software; you can redistribute it and/or
@@ -544,6 +544,11 @@ void scheme_get_outof_line(Scheme_Channel_Syncer *ch_w)
   get_outof_line((Scheme_Sema *)ch_w->obj, ch_w);
 }
 
+void scheme_get_back_into_line(Scheme_Channel_Syncer *ch_w)
+{
+  get_into_line((Scheme_Sema *)ch_w->obj, ch_w);
+}
+
 static int try_channel(Scheme_Sema *sema, Syncing *syncing, int pos, Scheme_Object **result)
 {
   if (SCHEME_CHANNELP(sema)) {
