@@ -1,6 +1,6 @@
 /*
   Racket
-  Copyright (c) 2004-2014 PLT Design Inc.
+  Copyright (c) 2004-2015 PLT Design Inc.
   Copyright (c) 1995-2001 Matthew Flatt
 
     This library is free software; you can redistribute it and/or
@@ -905,6 +905,60 @@ apply_values_sfs(Scheme_Object *data, SFS_Info *info)
   return data;
 }
 
+static Scheme_Object *with_immed_mark_sfs(Scheme_Object *o, SFS_Info *info)
+{
+  Scheme_With_Continuation_Mark *wcm = (Scheme_With_Continuation_Mark *)o;
+  Scheme_Object *k, *v, *b, *vec;
+  int pos, save_mnt;
+  
+  scheme_sfs_start_sequence(info, 3, 1);
+
+  k = scheme_sfs_expr(wcm->key, info, -1);
+  v = scheme_sfs_expr(wcm->val, info, -1);
+
+  scheme_sfs_push(info, 1, 1);
+
+  pos = info->stackpos;
+  save_mnt = info->max_nontail;
+
+  if (!info->pass) {
+    vec = scheme_make_vector(3, NULL);
+    scheme_sfs_save(info, vec);
+  } else {
+    vec = scheme_sfs_next_saved(info);
+    if (SCHEME_VEC_SIZE(vec) != 3)
+      scheme_signal_error("internal error: bad vector length");
+    info->max_used[pos] = SCHEME_INT_VAL(SCHEME_VEC_ELS(vec)[0]);
+    info->max_calls[pos] = SCHEME_INT_VAL(SCHEME_VEC_ELS(vec)[1]);
+    info->max_nontail = SCHEME_INT_VAL(SCHEME_VEC_ELS(vec)[2]);
+  }
+  
+  b = scheme_sfs_expr(wcm->body, info, -1);
+  
+  wcm->key = k;
+  wcm->val = v;
+  wcm->body = b;
+
+# if MAX_SFS_CLEARING
+  if (!info->pass)
+    info->max_nontail = info->ip;
+# endif
+
+  if (!info->pass) {
+    int n;
+    info->max_calls[pos] = info->max_nontail;
+    n = info->max_used[pos];
+    SCHEME_VEC_ELS(vec)[0] = scheme_make_integer(n);
+    n = info->max_calls[pos];
+    SCHEME_VEC_ELS(vec)[1] = scheme_make_integer(n);
+    SCHEME_VEC_ELS(vec)[2] = scheme_make_integer(info->max_nontail);
+  } else {
+    info->max_nontail = save_mnt;
+  }
+
+  return o;
+}
+
 static Scheme_Object *
 case_lambda_sfs(Scheme_Object *expr, SFS_Info *info)
 {
@@ -969,6 +1023,44 @@ static Scheme_Object *bangboxenv_sfs(Scheme_Object *data, SFS_Info *info)
   }
 }
 
+static Scheme_Object *flatten_begin0(Scheme_Object *o)
+{
+  /* At this point, we sometimes have (begin0 (begin0 (begin0  ...) ...)).
+     Flatten those out. */
+  Scheme_Sequence *s = (Scheme_Sequence *)o, *s2;
+  int i, extra = 0;
+
+  o = s->array[0];
+
+  while (SAME_TYPE(SCHEME_TYPE(o), scheme_begin0_sequence_type)) {
+    s2 = (Scheme_Sequence *)o;
+    extra += s2->count - 1;
+    o = s2->array[0];
+  }
+
+  if (extra) {
+    s2 = scheme_malloc_sequence(s->count + extra);
+    s2->so.type = scheme_begin0_sequence_type;
+    s2->count = s->count + extra;
+
+    extra = s2->count -1;
+    o = (Scheme_Object *)s;
+    while (SAME_TYPE(SCHEME_TYPE(o), scheme_begin0_sequence_type)) {
+      s = (Scheme_Sequence *)o;
+      for (i = s->count - 1; i ; i--) {
+        s2->array[extra--] = s->array[i];
+      }
+      o = s->array[i];
+    }
+    s2->array[extra--] = o;
+
+    if (extra != -1) scheme_signal_error("internal error: flatten begin0 failed");
+
+    return (Scheme_Object *)s2;
+  } else
+    return (Scheme_Object *)s;
+}
+
 static Scheme_Object *
 begin0_sfs(Scheme_Object *obj, SFS_Info *info)
 {
@@ -984,6 +1076,9 @@ begin0_sfs(Scheme_Object *obj, SFS_Info *info)
     ((Scheme_Sequence *)obj)->array[i] = le;
   }
 
+  if (info->pass)
+    obj = flatten_begin0(obj);
+
   return obj;
 }
 
@@ -1336,6 +1431,9 @@ Scheme_Object *scheme_sfs_expr(Scheme_Object *expr, SFS_Info *info, int closure_
   case scheme_apply_values_type:
     expr = apply_values_sfs(expr, info);
     break;
+  case scheme_with_immed_mark_type:
+    expr = with_immed_mark_sfs(expr, info);
+    break;
   case scheme_case_lambda_sequence_type:
     expr = case_lambda_sfs(expr, info);
     break;
