@@ -1,6 +1,6 @@
 /*
   Racket
-  Copyright (c) 2004-2014 PLT Design Inc.
+  Copyright (c) 2004-2015 PLT Design Inc.
   Copyright (c) 1995-2001 Matthew Flatt
 
     This library is free software; you can redistribute it and/or
@@ -129,11 +129,13 @@ scheme_init_type ()
   set_name(scheme_begin0_sequence_type, "<begin0-code>");
   set_name(scheme_splice_sequence_type, "<splicing-begin-code>");
   set_name(scheme_module_type, "<module-code>");
+  set_name(scheme_inline_variant_type, "<inline-variant-code>");
   set_name(scheme_set_bang_type, "<set!-code>");
   set_name(scheme_boxenv_type, "<boxenv-code>");
   set_name(scheme_require_form_type, "<require-code>");
   set_name(scheme_varref_form_type, "<varref-code>");
   set_name(scheme_apply_values_type, "<apply-values-code>");
+  set_name(scheme_with_immed_mark_type, "<with-immediate-mark-code>");
   set_name(scheme_case_lambda_sequence_type, "<case-lambda-code>");
 
   set_name(scheme_let_value_type, "<let-value-code>");
@@ -197,6 +199,11 @@ scheme_init_type ()
   set_name(scheme_channel_put_type, "<channel-put>");
   set_name(scheme_hash_table_type, "<hash>");
   set_name(scheme_hash_tree_type, "<hash>");
+  set_name(scheme_eq_hash_tree_type, "<hash>");
+  set_name(scheme_eqv_hash_tree_type, "<hash>");
+  set_name(scheme_hash_tree_indirection_type, "<hash>");
+  set_name(scheme_hash_tree_subtree_type, "<hash-node>");
+  set_name(scheme_hash_tree_collision_type, "<hash-node>");
   set_name(scheme_bucket_table_type, "<hash>");
   set_name(scheme_module_registry_type, "<module-registry>");
   set_name(scheme_case_closure_type, "<procedure>");
@@ -215,7 +222,9 @@ scheme_init_type ()
   set_name(scheme_will_executor_type, "<will-executor>");
   set_name(scheme_random_state_type, "<pseudo-random-generator>");
   set_name(scheme_regexp_type, "<regexp>");
-  set_name(scheme_rename_table_type, "<rename-table>");
+  set_name(scheme_scope_table_type, "<scope-table>");
+  set_name(scheme_propagate_table_type, "<propagate-table>");
+  set_name(scheme_scope_type, "<scope>");
   set_name(scheme_bucket_type, "<hash-table-bucket>");
   set_name(scheme_prefix_type, "<runtime-prefix>");
   set_name(scheme_resolve_prefix_type, "<resolve-prefix>");
@@ -566,6 +575,7 @@ void scheme_register_traversers(void)
   GC_REG_TRAV(scheme_begin_for_syntax_type, vector_obj);
   GC_REG_TRAV(scheme_varref_form_type, twoptr_obj);
   GC_REG_TRAV(scheme_apply_values_type, twoptr_obj);
+  GC_REG_TRAV(scheme_with_immed_mark_type, with_cont_mark);
   GC_REG_TRAV(scheme_boxenv_type, twoptr_obj);
   GC_REG_TRAV(scheme_case_lambda_sequence_type, case_closure);
   GC_REG_TRAV(scheme_begin0_sequence_type, seq_rec);
@@ -711,6 +721,7 @@ void scheme_register_traversers(void)
   GC_REG_TRAV(scheme_global_ref_type, twoptr_obj);
 
   GC_REG_TRAV(scheme_delay_syntax_type, small_object);
+  GC_REG_TRAV(scheme_marshal_share_type, small_object);
 
   GC_REG_TRAV(scheme_resolved_module_path_type, small_object);
 
@@ -719,8 +730,6 @@ void scheme_register_traversers(void)
 
   GC_REG_TRAV(scheme_rt_runstack, runstack_val);
 
-  GC_REG_TRAV(scheme_free_id_info_type, vector_obj);
-
   GC_REG_TRAV(scheme_rib_delimiter_type, small_object);
   GC_REG_TRAV(scheme_noninline_proc_type, small_object);
   GC_REG_TRAV(scheme_prune_context_type, small_object);
